package org.rcisoft.core.aop;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.rcisoft.common.component.CyGlobal;
import org.rcisoft.core.doc.anno.CyDoc;
import org.rcisoft.core.doc.enums.CyDocEnum;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.result.CyResultSystemExceptionEnums;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

/**
 * Created with family.
 * author: cy
 * Date: 18/10/25
 * Time: 下午8:06
 * description:  对称解密参数，防止报文被篡改
 *
 *
 * Aopconfig.java  使用
 *
 *           @Bean
 *           public CyDecryptParamAspect cyDecryptParamAspect(){
 *               return new CyDecryptParamAspect();
 *           }
 */
@Aspect
@Slf4j
@CyDoc(CyDocEnum.USELESS)
public class CyDecryptParamAspect {

    @Autowired
    private CyGlobal cyGlobal;





    /**
     * 解密拦截
     * @param proceedingJoinPoint
     * @return
     * @throws Throwable
     */
    @Around("@annotation(org.rcisoft.core.anno.CyDecryptEnAbled)")
    public Object decryptParameter(ProceedingJoinPoint proceedingJoinPoint) throws Throwable {
        return dealWithParameterDecrypted(proceedingJoinPoint);
    }

    /**
     * 处理
     * @param proceedingJoinPoint
     * @return
     * @throws Throwable
     */
    private Object dealWithParameterDecrypted(ProceedingJoinPoint proceedingJoinPoint) throws Throwable {
        HttpServletRequest request = ((ServletRequestAttributes) (RequestContextHolder.getRequestAttributes())).getRequest();
        Map<String, String> param = new HashMap<String, String>();
        Enumeration penum = (Enumeration) request.getParameterNames();
        //将所有除sign与文件的参数放入param
        while (penum.hasMoreElements()) {
            String pKey = (String) penum.nextElement();
            String value = request.getParameter(pKey);
            //sign和uploadFile不参与 值为空也不参与
            if (!"sign".equals(pKey) && !"uploadFile".equals(pKey)
                    && StringUtils.isNotBlank(value)) {
                param.put(pKey, value);
            }
        }
        StringBuilder str = new StringBuilder();
        for (Map.Entry e :
                param.entrySet()) {
            str.append(e.getKey());
            str.append("=");
            str.append(e.getValue());
            str.append("&");
        }
        str.append("key=" + cyGlobal.getDecryptKey());
        //加密并大写
        String decrypt = this.md5Encode(str.toString(), "utf-8").toUpperCase();
        //比较加密的数据与前台发来的sign是否一致
        if (!decrypt.equals(request.getParameter("sign"))) {
            throw new CyServiceException(CyResultSystemExceptionEnums.PARAMETER_DECRYPT_ERROR);
        }
        return proceedingJoinPoint.proceed();
    }




    /**
     * 加密
     * @param origin
     * @param charsetName
     * @return
     */
    public static String md5Encode(String origin, String charsetName) throws UnsupportedEncodingException, NoSuchAlgorithmException {
        String resultString = null;
        try {
            resultString = new String(origin);
            MessageDigest md = MessageDigest.getInstance("MD5");
            if (StringUtils.isBlank(charsetName)) {
                resultString = byteArrayToHexString(md.digest(resultString.getBytes()));
            } else {
                resultString = byteArrayToHexString(md.digest(resultString.getBytes(charsetName)));
            }
        } catch (Exception e) {
            throw e;
        }
        return resultString;
    }

    public static String byteArrayToHexString(byte b[]) {
        StringBuffer resultSb = new StringBuffer();
        for (int i = 0; i < b.length; i++) {
            resultSb.append(byteToHexString(b[i]));
        }
        return resultSb.toString();
    }

    public static String byteToHexString(byte b) {
        int n = b;
        if (n < 0) {
            n += 256;
        }
        int d1 = n / 16;
        int d2 = n % 16;
        return hexDigits[d1] + hexDigits[d2];
    }

    public static final String hexDigits[] = {"0", "1", "2", "3", "4", "5",
            "6", "7", "8", "9", "a", "b", "c", "d", "e", "f"};
}
